module usb3_top_host
(
    input           sys_clk,
	input			key_resetn,
    // input           phy_resetn,
    output  [39:0]      tx_data_8b10b_o,
    input   [39:0]      rx_data_8b10b_i,

    output          RxElecIdle1,
    output          led1,
    output          led2
);

reg     [7:0]   cnt0 = 0;
reg     [15:0]  cnt1 = 0;
reg     [15:0]  cnt2 = 0;
reg     [31:0]  cnt3 = 0;
reg     [31:0]  cnt4 = 0;
reg     [7:0]   cnt5 = 0;
reg		[7:0]	key_resetn_r = 8'b11111111;
wire            phy_resetn;
wire            phy_resetn_o;
wire            pclk;
reg				led1_r = 1'b0;
reg				led2_r = 1'b0;

wire	[2:0]	RxStatus;
wire			PhyStatus;
wire			phy_pipe_half_clk;        // 125MHz:  1/2 PCLK
wire			phy_pipe_half_clk_phase;  // 125MHz:  1/2 PCLK, phase shift 90
wire			phy_pipe_quarter_clk;     // 62.5MHz: 1/4 PCLK
wire	[31:0]	phy_pipe_rx_data;
wire	[3:0]	phy_pipe_rx_datak;
wire	[1:0]	phy_pipe_rx_valid;
wire			phy_pipe_rx_valid_o;
wire	[31:0]	phy_pipe_tx_data;
wire	[3:0]	phy_pipe_tx_datak;
wire			phy_reset_n;
wire			phy_out_enable;
wire			phy_phy_reset_n;
wire			phy_tx_detrx_lpbk;
wire			phy_tx_elecidle;
wire			phy_rx_elecidle;
wire	[5:0]	phy_rx_status;
wire	[1:0]	phy_power_down;
wire	[1:0]	phy_phy_status_i;
wire	        phy_phy_status_o;
wire			phy_pwrpresent;
wire			phy_tx_oneszeros;
wire	[1:0]	phy_tx_deemph;
wire	[2:0]	phy_tx_margin;
wire			phy_tx_swing;
wire			phy_rx_polarity;
wire			phy_rx_termination;
wire			phy_rate;
wire			phy_elas_buf_mode;

assign led1 = led1_r;
assign led2 = led2_r;
assign phy_resetn = cnt0 > 127? 1'b1 : 1'b0;

always@(posedge sys_clk)
begin
	key_resetn_r <= {key_resetn_r[6:0], key_resetn};
end

always@(posedge sys_clk or negedge key_resetn_r[7])
begin
	if(!key_resetn_r[7])
	begin
		cnt3	<= 0;
		led1_r	<= 0;
	end
	else
	begin
		if(cnt3 < 50000000)
		begin
			cnt3	<= cnt3 + 1'b1;
		end
		else
		begin
			cnt3	<= 32'd0;
			led1_r	<= ~led1_r;
		end
	end
end

always@(posedge pclk)
begin
	if(cnt4 < 125000000)
	begin
		cnt4	<= cnt4 + 1'b1;
	end
	else
	begin
		cnt4	<= 32'd0;
		led2_r	<= ~led2_r;
	end
end

always@(posedge sys_clk)
begin
    if(cnt0 < 255)
    begin
        cnt0                <= cnt0 + 1'b1;
    end
    else
    begin
        cnt0                <= cnt0;
    end
end

//usb3_0_phy_top Inst_usb3_0_phy_top(
//	// .ref_clk                (sys_clk),//input			    
//	.phy_resetn             (phy_resetn_o),//input			    
//	.sys_clk				(sys_clk),
//	.pclk                   (pclk),//output			    
//	.PipeTxData             (phy_pipe_tx_data),//input       [31:0]  
//    .PipeTxDataK            (phy_pipe_tx_datak),//input       [3:0]   
//    .PipeRxData             (phy_pipe_rx_data),//output      [31:0]  
//    .PipeRxDataK            (phy_pipe_rx_datak),//output      [3:0]   
//    .PipeRxDataValid        (phy_pipe_rx_valid_o),//output              
//    .TxDetectRx_loopback    (phy_tx_detrx_lpbk),//input               
//    .TxElecIdle             (phy_tx_elecidle),//input               
//    // .TxDeemph               (2'b0),//input   [1:0]       
//    // .TxMargin               (3'b0),//input   [2:0]       
//    // .TxSwing                (1'b0),//input               
//    .RxPolarity             (phy_rx_polarity),//input               
//    .RxEqTraining           (1'b0),//input               
//    .RxTermination          (phy_rx_termination),//input               
//    .RxElecIdle             (phy_rx_elecidle),//output              
//    .RxStatus               (RxStatus),//output  [2:0]       
//    .ElasticityBufferMode   (phy_elas_buf_mode),//input               
//    .TxOnesZeros            (phy_tx_oneszeros),//input               
//    .PowerDown              (phy_power_down),//input   [1:0]       
//    // .Rate                   (1'b0),//input               
//    .PhyStatus              (PhyStatus),//output              
//    .PowerPresent           (phy_pwrpresent),//output              
//	.tx_data_8b10b_o		(tx_data_8b10b_o),
//	.rx_data_8b10b_i		(rx_data_8b10b_i)
//);


SerDes_Top SerDes_Top_inst_host(
	.sys_clk_i ( sys_clk ),
    .USB3_0_PHY_Top_pclk(pclk), //output USB3_0_PHY_Top_pclk
    .USB3_0_PHY_Top_PipeRxData(phy_pipe_rx_data), //output [31:0] USB3_0_PHY_Top_PipeRxData
    .USB3_0_PHY_Top_PipeRxDataK(phy_pipe_rx_datak), //output [3:0] USB3_0_PHY_Top_PipeRxDataK
    .USB3_0_PHY_Top_PipeRxDataValid(phy_pipe_rx_valid_o), //output USB3_0_PHY_Top_PipeRxDataValid
    .USB3_0_PHY_Top_RxElecIdle(phy_rx_elecidle), //output USB3_0_PHY_Top_RxElecIdle
    .USB3_0_PHY_Top_RxStatus(RxStatus), //output [2:0] USB3_0_PHY_Top_RxStatus
    .USB3_0_PHY_Top_PhyStatus(PhyStatus), //output USB3_0_PHY_Top_PhyStatus
    .USB3_0_PHY_Top_PowerPresent(phy_pwrpresent), //output USB3_0_PHY_Top_PowerPresent
    .USB3_0_PHY_Top_phy_resetn(phy_resetn_o), //input USB3_0_PHY_Top_phy_resetn
    .USB3_0_PHY_Top_PipeTxData(phy_pipe_tx_data), //input [31:0] USB3_0_PHY_Top_PipeTxData
    .USB3_0_PHY_Top_PipeTxDataK(phy_pipe_tx_datak), //input [3:0] USB3_0_PHY_Top_PipeTxDataK
    .USB3_0_PHY_Top_TxDetectRx_loopback(phy_tx_detrx_lpbk), //input USB3_0_PHY_Top_TxDetectRx_loopback
    .USB3_0_PHY_Top_TxElecIdle(phy_tx_elecidle), //input USB3_0_PHY_Top_TxElecIdle
    .USB3_0_PHY_Top_RxPolarity(phy_rx_polarity), //input USB3_0_PHY_Top_RxPolarity
    .USB3_0_PHY_Top_RxEqTraining(1'b0), //input USB3_0_PHY_Top_RxEqTraining
    .USB3_0_PHY_Top_RxTermination(phy_rx_termination), //input USB3_0_PHY_Top_RxTermination
    .USB3_0_PHY_Top_TxOnesZeros(phy_tx_oneszeros), //input USB3_0_PHY_Top_TxOnesZeros
    .USB3_0_PHY_Top_PowerDown(phy_power_down), //input [1:0] USB3_0_PHY_Top_PowerDown
    .USB3_0_PHY_Top_ElasticityBufferMode(phy_elas_buf_mode), //input USB3_0_PHY_Top_ElasticityBufferMode     
	.tx_data_8b10b_o		(tx_data_8b10b_o),
	.rx_data_8b10b_i		(rx_data_8b10b_i)
);

assign phy_rx_status = {3'b000, RxStatus};
assign phy_phy_status_i = {1'b0, PhyStatus};
assign phy_pipe_rx_valid = {phy_pipe_rx_valid_o, phy_pipe_rx_valid_o};






usb3_top_tusb1310a_host Inst_usb3_top_tusb1310a_host (
	.ext_clk					(pclk),//input	wire			
	.reset_n					(phy_resetn & key_resetn_r[7]),//input	wire			
	.phy_pipe_half_clk			(pclk),//input	wire			        // 125MHz:  1/2 PCLK
	.phy_pipe_half_clk_phase	(pclk),//input	wire			  // 125MHz:  1/2 PCLK, phase shift 90
	.phy_pipe_quarter_clk		(pclk),//input	wire			     // 62.5MHz: 1/4 PCLK
	.phy_pipe_rx_data			(phy_pipe_rx_data),//input	wire	[31:0]	
	.phy_pipe_rx_datak			(phy_pipe_rx_datak),//input	wire	[3:0]	
	.phy_pipe_rx_valid			(phy_pipe_rx_valid),//input	wire	[1:0]	
	.phy_pipe_tx_data			(phy_pipe_tx_data),//output	wire	[31:0]	
	.phy_pipe_tx_datak			(phy_pipe_tx_datak),//output	wire	[3:0]	
	.phy_reset_n				(phy_resetn_o),//output	wire			
	.phy_out_enable				(),//output	wire			
	.phy_phy_reset_n			(),//output	wire			
	.phy_tx_detrx_lpbk			(phy_tx_detrx_lpbk),//output	wire			
	.phy_tx_elecidle			(phy_tx_elecidle	),//output	wire			
	.phy_rx_elecidle			(phy_rx_elecidle	),//inout	wire			
	.phy_rx_status				(phy_rx_status		),//input	wire	[5:0]	
	.phy_power_down				(phy_power_down		),//output	wire	[1:0]	
	.phy_phy_status_i			(phy_phy_status_i	),//input   wire	[1:0]	
	.phy_phy_status_o			(phy_phy_status_o	),//output	wire	        
	.phy_pwrpresent				(phy_pwrpresent		),//input	wire			
	.phy_tx_oneszeros			(phy_tx_oneszeros	),//output	wire			
	.phy_tx_deemph				(phy_tx_deemph		),//output	wire	[1:0]	
	.phy_tx_margin				(phy_tx_margin		),//output	wire	[2:0]	
	.phy_tx_swing				(phy_tx_swing		),//output	wire			
	.phy_rx_polarity			(phy_rx_polarity	),//output	wire			
	.phy_rx_termination			(phy_rx_termination	),//output	wire			
	.phy_rate					(phy_rate			),//output	wire			
	.phy_elas_buf_mode			(phy_elas_buf_mode	),//output	wire			
	.buf_in_addr				(),//input	wire	[8:0]	
	.buf_in_data				(),//input	wire	[31:0]	
	.buf_in_wren				(),//input	wire			
	.buf_in_request				(),//output	wire			
	.buf_in_ready				(),//output	wire			
	.buf_in_commit				(),//input	wire			
	.buf_in_commit_len			(),//input	wire	[10:0]	
	.buf_in_commit_ack			(),//output	wire			
	.buf_out_addr				(),//input	wire	[8:0]	
	.buf_out_q					(),//output	wire	[31:0]	
	.buf_out_len				(),//output	wire	[10:0]	
	.buf_out_hasdata			(),//output	wire			
	.buf_out_arm				(),//input	wire			
	.buf_out_arm_ack			(),//output	wire			
	.vend_req_act				(),//output	wire			
	.vend_req_request			(),//output	wire	[7:0]	
	.vend_req_val				(),//output	wire	[15:0]	
	.dbg_pipe_state				(),//output  wire    [5:0] 	
	.dbg_ltssm_state			() //output  wire    [4:0] 	
);




endmodule